//
// Simple sky vertex shader with a procedural tow clouds layers and day/night change
//
// Author: Alex V. Boreskoff
//

varying	vec3  pos;
varying	vec3  v;
varying	vec3  l;
varying	vec3  colorYxy;
varying	float lv;						// dot (  l, v )
varying vec3  skyVec;					// rotated vector to sky

uniform	float		time;
uniform float		sunAngle;
uniform	sampler3D	noiseMap;
uniform samplerCube	skyMap;

vec3	convertColor ()
{
	vec3	clrYxy = vec3 ( colorYxy );
													// now rescale Y component
	clrYxy [0] = 1.0 - exp ( -clrYxy [0] / 25.0 );

	float	ratio    = clrYxy [0] / clrYxy [2];		// Y / y = X + Y + Z
	vec3	XYZ;

	XYZ.x = clrYxy [1] * ratio;						// X = x * ratio
	XYZ.y = clrYxy [0];								// Y = Y
	XYZ.z = ratio - XYZ.x - XYZ.y;					// Z = ratio - X - Y

	const	vec3	rCoeffs = vec3 ( 3.240479, -1.53715, -0.49853  );
	const	vec3	gCoeffs = vec3 ( -0.969256, 1.875991, 0.041556 );
	const	vec3	bCoeffs = vec3 ( 0.055684, -0.204043, 1.057311 );

	return	vec3 ( dot ( rCoeffs, XYZ ), dot ( gCoeffs, XYZ ), dot ( bCoeffs, XYZ ) );
}

vec3	clouds ( const vec3 tex, float t, const vec2 vel )
{
    vec3 	dt   = vec3 ( vel, 0.061 ) * t;
    vec3 	tex1 = vec3 ( tex.xy, 0.234753 ) + dt; 
	vec3	n1   = texture3D ( noiseMap, tex1       ).xyz / 2.0;
	vec3	n2   = texture3D ( noiseMap, tex1 * 2.0 ).xyz / 4.0;
	vec3	n3   = texture3D ( noiseMap, tex1 * 4.0 ).xyz / 8.0;
	vec3	n4   = texture3D ( noiseMap, tex1 * 8.0 ).xyz / 16.0;
	
	return (n1 + n2 + n3 + n4) / (0.5 + 0.25 + 0.125 + 0.0625 );		// return normalized sum
}

float	density ( float v, float d  )
{
	return clamp ( (1.0 + d)*pow ( v, 4.0 ) - 0.3, 0.0, 1.0 );
}

void main ()
{
	const	vec4	cloudColor     = vec4 ( 1.0, 1.0, 1.0, 1.0 );
	const	float	cloudDensity1  = 3.5;

	vec3	n1  = clouds ( 0.6*pos, 0.0631*time, vec2 ( 0.6, 0.2 ) ); 
	
										// now compute clouds based on n
	vec4	skyColor = vec4	( clamp ( convertColor (), 0.0, 1.0 ), 1.0 );
	float	alpha    = 0.75 * ( 1.2 - lv );

	skyColor += alpha*textureCube ( skyMap, skyVec );

	gl_FragColor = mix ( skyColor, cloudColor, density ( n1.x, cloudDensity1 ) );
}

